/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.introspection.items;

import static java.util.Collections.emptyList;
import static java.util.Collections.emptySet;
import static java.util.Collections.unmodifiableList;
import static java.util.Collections.unmodifiableSet;

import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;

/**
 * Base class for {@link IIntrospectionDetailsItem}s with class-based handler registrations.
 * <P>
 * The handler key set and the returned handler lists are read-only.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 5CA71612B333D110BBBD0A6EEC6A977B
 */
public abstract class EObjectAware2IntrospectionDetailsItemBase<T extends Object> implements
		IIntrospectionDetailsItem {

	/** Read-only copy of the services handler list. */
	protected final Map<Class<?>, Map<Class<?>, List<T>>> handlerMap;

	/** Constructor. */
	public EObjectAware2IntrospectionDetailsItemBase(
			Map<Class<?>, Map<Class<?>, List<T>>> handlerMap) {
		this.handlerMap = handlerMap;
	}

	/** Returns the first registration classes. */
	public Collection<Class<?>> getFirstHandlerKeyClasses() {
		return unmodifiableSet(handlerMap.keySet());
	}

	/** Returns the second registration classes. */
	public Collection<Class<?>> getSecondHandlerKeyClasses(Class<?> regClass) {
		Map<Class<?>, List<T>> map = handlerMap.get(regClass);
		if(map != null) {
			return unmodifiableSet(map.keySet());
		}
		return emptySet();
	}

	/** Returns the handler list for the given registration class. */
	public List<T> getHandlerList(Class<?> firstRegClass, Class<?> secondRegClass) {
		Map<Class<?>, List<T>> map = handlerMap.get(firstRegClass);
		if(map != null) {
			List<T> l = map.get(secondRegClass);
			return unmodifiableList(l);
		}
		return emptyList();
	}

	/** Counts all unique handlers. */
	public final int countHandlers() {
		Set<T> handlerSet = new HashSet<T>();
		for(Class<?> c : handlerMap.keySet()) {
			Map<Class<?>, List<T>> map = handlerMap.get(c);
			if(map == null) {
				continue;
			}
			for(Class<?> c2 : map.keySet()) {
				List<T> hlist = map.get(c2);
				if(hlist == null) {
					continue;
				}
				handlerSet.addAll(hlist);
			}
		}
		return handlerSet.size();
	}
}
